@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2000-2008 3APA3A
- *
- * please read License Agreement
- *
- */
+   (c) 2002-2016 by Vladimir Dubrovin <3proxy@3proxy.ru>
+
+   please read License Agreement
+
+*/
 
 #include "proxy.h"
 
@@ -365,28 +365,11 @@ static void * ef_ace_next(struct node * node){
 	return ((struct ace *)node->value) -> next;
 }
 
+
+char * aceaction (int action);
+
 static void * ef_ace_type(struct node * node){
-	switch (((struct ace *)node->value) -> action) {
-		case ALLOW:
-		case REDIRECT:
-			return "allow";
-		case DENY:
-			return "deny";
-		case BANDLIM:
-			return "bandlim";
-		case NOBANDLIM:
-			return "nobandlim";
-		case COUNTIN:
-			return "countin";
-		case NOCOUNTIN:
-			return "nocountin";
-		case COUNTOUT:
-			return "countout";
-		case NOCOUNTOUT:
-			return "nocountout";
-		default:
-			return "unknown";
-	}
+	return aceaction(((struct ace *)node->value) -> action);
 }
 
 
@@ -520,6 +503,9 @@ static void * ef_server_childcount(struct node * node){
 
 static void * ef_server_log(struct node * node){
 	if(((struct srvparam *)node->value) -> logfunc == lognone)	return "none";
+#ifndef NORADIUS
+	else if(((struct srvparam *)node->value) -> logfunc == logradius)	return "radius";
+#endif
 	else if(((struct srvparam *)node->value) -> logfunc == logstdout)
 		return (((struct srvparam *)node->value) -> logtarget)?"file":"stdout";
 #ifndef _WIN32
@@ -648,6 +634,14 @@ static void * ef_client_threadid(struct node * node){
 	return &((struct clientparam *)node->value) -> threadid;
 }
 
+static void * ef_client_clisock(struct node * node){
+	return &((struct clientparam *)node->value) -> clisock;
+}
+
+static void * ef_client_remsock(struct node * node){
+	return &((struct clientparam *)node->value) -> remsock;
+}
+
 static void * ef_client_starttime(struct node * node){
 	return &((struct clientparam *)node->value) -> time_start;
 }
@@ -785,6 +779,8 @@ static struct property prop_client[] = {
 	{prop_client + 17, "maxtrafin", ef_client_maxtrafin64, TYPE_UNSIGNED64, "maximum traffic allowed for download"},
 	{prop_client + 18, "maxtrafout", ef_client_maxtrafout64, TYPE_UNSIGNED64, "maximum traffic allowed for upload"},
 	{prop_client + 19, "pwtype", ef_client_pwtype, TYPE_INTEGER, "type of client password"},
+	{prop_client + 20, "clisock", ef_client_clisock, TYPE_INTEGER, "client socket"},
+	{prop_client + 21, "remsock", ef_client_remsock, TYPE_INTEGER, "remote socket"},
 	{NULL, "next", ef_client_next, TYPE_CLIENT, "next"}
 
 	
