@@ -1,6 +1,6 @@
 /*
    3APA3A simpliest proxy server
-   (c) 2002-2008 by ZARAZA <3APA3A@security.nnov.ru>
+   (c) 2002-2016 by Vladimir Dubrovin <3proxy@3proxy.ru>
 
    please read License Agreement
 
@@ -13,49 +13,49 @@
 
 char * proxy_stringtable[] = {
 /* 0 */	"HTTP/1.0 400 Bad Request\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>400 Bad Request</title></head>\r\n"
 	"<body><h2>400 Bad Request</h2></body></html>\r\n",
 
 /* 1 */	"HTTP/1.0 502 Bad Gateway\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>502 Bad Gateway</title></head>\r\n"
 	"<body><h2>502 Bad Gateway</h2><h3>Host Not Found or connection failed</h3></body></html>\r\n",
 
 /* 2 */	"HTTP/1.0 503 Service Unavailable\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>503 Service Unavailable</title></head>\r\n"
 	"<body><h2>503 Service Unavailable</h2><h3>You have exceeded your traffic limit</h3></body></html>\r\n",
 
 /* 3 */	"HTTP/1.0 503 Service Unavailable\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>503 Service Unavailable</title></head>\r\n"
 	"<body><h2>503 Service Unavailable</h2><h3>Recursion detected</h3></body></html>\r\n",
 
 /* 4 */	"HTTP/1.0 501 Not Implemented\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>501 Not Implemented</title></head>\r\n"
 	"<body><h2>501 Not Implemented</h2><h3>Required action is not supported by proxy server</h3></body></html>\r\n",
 
 /* 5 */	"HTTP/1.0 502 Bad Gateway\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>502 Bad Gateway</title></head>\r\n"
 	"<body><h2>502 Bad Gateway</h2><h3>Failed to connect parent proxy</h3></body></html>\r\n",
 
 /* 6 */	"HTTP/1.0 500 Internal Error\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>500 Internal Error</title></head>\r\n"
@@ -63,7 +63,7 @@ char * proxy_stringtable[] = {
 
 /* 7 */	"HTTP/1.0 407 Proxy Authentication Required\r\n"
 	"Proxy-Authenticate: Basic realm=\"proxy\"\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>407 Proxy Authentication Required</title></head>\r\n"
@@ -75,14 +75,14 @@ char * proxy_stringtable[] = {
 	"Content-Type: text/html\r\n\r\n",
 
 /* 10*/	"HTTP/1.0 404 Not Found\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>404 Not Found</title></head>\r\n"
 	"<body><h2>404 Not Found</h2><h3>File not found</body></html>\r\n",
 	
 /* 11*/	"HTTP/1.0 403 Forbidden\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>403 Access Denied</title></head>\r\n"
@@ -92,34 +92,34 @@ char * proxy_stringtable[] = {
 #ifndef NOCRYPT
 	"Proxy-Authenticate: NTLM\r\n"
 #endif
-	"Proxy-Authenticate: basic realm=\"proxy\"\r\n"
-	"Proxy-Connection: close\r\n"
+	"Proxy-Authenticate: Basic realm=\"proxy\"\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>407 Proxy Authentication Required</title></head>\r\n"
 	"<body><h2>407 Proxy Authentication Required</h2><h3>Access to requested resource disallowed by administrator or you need valid username/password to use this resource</h3></body></html>\r\n",
 
 /* 13*/	"HTTP/1.0 407 Proxy Authentication Required\r\n"
-	"Proxy-Connection: keep-alive\r\n"
+	"Connection: keep-alive\r\n"
 	"Content-Length: 0\r\n"
 	"Proxy-Authenticate: NTLM ",
 
 /* 14*/	"HTTP/1.0 403 Forbidden\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<pre>",
 
 /* 15*/	"HTTP/1.0 503 Service Unavailable\r\n"
-	"Proxy-Connection: close\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>503 Service Unavailable</title></head>\r\n"
 	"<body><h2>503 Service Unavailable</h2><h3>Your request violates configured policy</h3></body></html>\r\n",
 
 /* 16*/	"HTTP/1.0 401 Authentication Required\r\n"
-	"WWW-Authenticate: basic realm=\"FTP Server\"\r\n"
-	"Proxy-Connection: close\r\n"
+	"WWW-Authenticate: Basic realm=\"FTP Server\"\r\n"
+	"Connection: close\r\n"
 	"Content-type: text/html; charset=utf-8\r\n"
 	"\r\n"
 	"<html><head><title>401 FTP Server requires authentication</title></head>\r\n"
@@ -152,7 +152,7 @@ static void logurl(struct clientparam * param, char * buf, char * req, int ftp){
 		strcpy(se, sb);
 	}
  }
- if(param->res != 555 && param->res != 508)(*param->srv->logfunc)(param, (unsigned char *)(req?buf:NULL));
+ if(param->res != 555 && param->res != 508)dolog(param, (unsigned char *)(req?buf:NULL));
 }
 
 void decodeurl(unsigned char *s, int allowcr){
@@ -231,16 +231,17 @@ void * proxychild(struct clientparam* param) {
  SOCKET ftps;
  char ftpbuf[FTPBUFSIZE];
  int inftpbuf = 0;
+ int haveconnection = 0;
 #ifndef WITHMAIN
  FILTER_ACTION action;
 #endif
 
 
 
- 
+ if(param->remsock != INVALID_SOCKET) haveconnection = 1; 
  if(!(buf = myalloc(BUFSIZE))) {RETURN(21);}
  bufsize = BUFSIZE;
- anonymous = param->srv->singlepacket;
+ anonymous = param->srv->anonymous;
 for(;;){
  memset(buf, 0, bufsize);
  inbuf = 0;
@@ -439,9 +440,11 @@ for(;;){
 		if(!sb)continue;
 		++sb;
 		while(isspace(*sb))sb++;
-		if(!strncasecmp((char *)sb,"keep-alive", 10))keepalive = 1;
-		else keepalive = 0;
-		continue; 
+		if(strncasecmp((char *)sb,"upgrade", 7)){
+			if(!strncasecmp((char *)sb,"keep-alive", 10))keepalive = 1;
+			else keepalive = 0;
+			continue; 
+		}
 	}
 	if( i > 11 && !strncasecmp((char *)(buf+inbuf),  "Expect: 100", 11)){
 		keepalive = 1;
@@ -535,17 +538,28 @@ for(;;){
 	RETURN(0);
  }
  if(action != PASS) RETURN(517);
- if(param->ndatfilterscli > 0 && contentlength64 > 0){
+ param->nolongdatfilter = 0;
+
+ 
+ if (conf.filtermaxsize && contentlength64 > (uint64_t)conf.filtermaxsize) {
+	param->nolongdatfilter = 1;
+ }
+ else if(param->ndatfilterscli > 0 && contentlength64 > 0){
   uint64_t newlen64;
   newlen64 = sockfillbuffcli(param, (unsigned long)contentlength64, CONNECTION_S);
   if(newlen64 == contentlength64) {
+	action = handlepredatflt(param);
+	if(action == HANDLED){
+		RETURN(0);
+	}
+	if(action != PASS) RETURN(19);
 	action = handledatfltcli(param,  &param->clibuf, (int *)&param->clibufsize, 0, (int *)&param->cliinbuf);
 	if(action == HANDLED){
 		RETURN(0);
 	}
 	if(action != PASS) RETURN(517);
 	contentlength64 = param->cliinbuf;
-	param->ndatfilterscli = 0;
+	param->nolongdatfilter = 1;
   }
   sprintf((char*)buf+strlen((char *)buf), "Content-Length: %"PRINTF_INT64_MODIFIER"u\r\n", contentlength64);
  }
@@ -553,7 +567,16 @@ for(;;){
 #endif
 
  if(param->srv->needuser > 1 && !param->username) {RETURN(4);}
- if((res = (*param->srv->authfunc)(param))) {RETURN(res);}
+ if((res = (*param->srv->authfunc)(param))) {
+	if (res <= 10 || haveconnection || param->transparent) RETURN(res);
+	so._closesocket(param->remsock);
+	param->remsock = INVALID_SOCKET;
+	param->redirected = 0;
+	param->redirtype = 0;
+	memset(&param->sinsl, 0, sizeof(param->sinsl));
+	memset(&param->sinsr, 0, sizeof(param->sinsr));
+	if((res = (*param->srv->authfunc)(param))) RETURN(res);
+ }
 
  if(ftp && param->redirtype != R_HTTP){
 	SOCKET s;
@@ -616,7 +639,7 @@ for(;;){
 		s = param->remsock;
 		param->remsock = ftps;
 		if((param->operation == FTP_PUT) && (contentlength64 > 0)) param->waitclient64 = contentlength64;
-		res = sockmap(param, conf.timeouts[CONNECTION_L]);
+		res = mapsocket(param, conf.timeouts[CONNECTION_L]);
 		if (res == 99) res = 0;
 		so._closesocket(ftps);
 		ftps = INVALID_SOCKET;
@@ -786,7 +809,7 @@ for(;;){
 				sprintf(ftpbuf, 
 					"HTTP/1.0 200 OK\r\n"
 					"Content-Type: text/html\r\n"
-					"Proxy-Connection: keep-alive\r\n"
+					"Connection: keep-alive\r\n"
 					"Content-Length: %d\r\n\r\n",
 					inbuf);
 				socksend(param->clisock, (unsigned char *)ftpbuf, (int)strlen(ftpbuf), conf.timeouts[STRING_S]);
@@ -802,9 +825,17 @@ for(;;){
 
  if(isconnect && param->redirtype != R_HTTP) {
 	socksend(param->clisock, (unsigned char *)proxy_stringtable[8], (int)strlen(proxy_stringtable[8]), conf.timeouts[STRING_S]);
-	if(param->redirectfunc) return (*param->redirectfunc)(param);
-	param->res =  sockmap(param, conf.timeouts[CONNECTION_L]);
-	if(param->redirectfunc) return (*param->redirectfunc)(param);
+	if(param->redirectfunc) {
+		if(req)myfree(req);
+		if(buf)myfree(buf);
+		return (*param->redirectfunc)(param);
+	}
+	param->res =  mapsocket(param, conf.timeouts[CONNECTION_L]);
+	if(param->redirectfunc) {
+		if(req)myfree(req);
+		if(buf)myfree(buf);
+		return (*param->redirectfunc)(param);
+	}
 	RETURN(param->res);
  }
 
@@ -813,6 +844,10 @@ for(;;){
  }
 
  else {
+#ifdef TCP_CORK
+	int opt = 1;
+	so._setsockopt(param->remsock, IPPROTO_TCP, TCP_CORK, (unsigned char *)&opt, sizeof(int));
+#endif
 	 redirect = 1;
 	 res = (int)strlen((char *)req);
 	 if(socksend(param->remsock, req , res, conf.timeouts[STRING_L]) != res) {
@@ -847,9 +882,11 @@ for(;;){
 		sprintf((char*)buf+strlen((char *)buf), "\r\n");
  }
 #endif
- if(keepalive <= 1) sprintf((char*)buf+strlen((char *)buf), "%s: %s\r\n", (param->redirtype == R_HTTP)?"Proxy-Connection":"Connection", keepalive? "keep-alive":"close");
+ if(keepalive <= 1) {
+	sprintf((char*)buf+strlen((char *)buf), "Connection: %s\r\n", keepalive? "keep-alive":"close");
+ }
  if(param->extusername){
-	sprintf((char*)buf + strlen((char *)buf), "%s: basic ", (redirect)?"Proxy-Authorization":"Authorization");
+	sprintf((char*)buf + strlen((char *)buf), "%s: Basic ", (redirect)?"Proxy-Authorization":"Authorization");
 	sprintf((char*)username, "%.128s:%.128s", param->extusername, param->extpassword?param->extpassword:(unsigned char*)"");
 	en64(username, buf+strlen((char *)buf), (int)strlen((char *)username));
 	sprintf((char*)buf + strlen((char *)buf), "\r\n");
@@ -858,15 +895,20 @@ for(;;){
  if ((res = socksend(param->remsock, buf+reqlen, (int)strlen((char *)buf+reqlen), conf.timeouts[STRING_S])) != (int)strlen((char *)buf+reqlen)) {
 	RETURN(518);
  }
+#ifdef TCP_CORK
+ {
+	int opt = 0;
+	so._setsockopt(param->remsock, IPPROTO_TCP, TCP_CORK, (unsigned char *)&opt, sizeof(int));
+ }
+#endif
  param->statscli64 += res;
  param->nwrites++;
  if(param->bandlimfunc) {
 	sleeptime = param->bandlimfunc(param, 0, (int)strlen((char *)buf));
  }
  if(contentlength64 > 0){
-	 param->nolongdatfilter = 0;
 	 param->waitclient64 = contentlength64;
-	 res = sockmap(param, conf.timeouts[CONNECTION_S]);
+	 res = mapsocket(param, conf.timeouts[CONNECTION_S]);
 	 param->waitclient64 = 0;
 	 if(res != 99) {
 		RETURN(res);
@@ -891,7 +933,7 @@ for(;;){
 		++sb;
 		while(isspace(*sb))sb++;
 		if(strncasecmp((char *)sb,"keep-alive", 10))ckeepalive = 0;
-		if(!param->srv->transparent)continue; 
+		if(!param->srv->transparent && res >= 200)continue; 
 	}
 	else if(i> 6 && !param->srv->transparent && (!strncasecmp((char *)(buf+inbuf), "proxy-", 6))){
 		continue; 
@@ -967,10 +1009,15 @@ for(;;){
  if (conf.filtermaxsize && contentlength64 > (uint64_t)conf.filtermaxsize) {
 	param->nolongdatfilter = 1;
  }
- else if(param->unsafefilter && param->ndatfilterssrv > 0 && contentlength64 > 0 && param->operation != HTTP_HEAD && res != 204 && res != 304){
+ else if(param->ndatfilterssrv > 0 && contentlength64 > 0 && param->operation != HTTP_HEAD && res != 204 && res != 304){
   uint64_t newlen;
   newlen = (uint64_t)sockfillbuffsrv(param, (unsigned long) contentlength64, CONNECTION_S);
   if(newlen == contentlength64) {
+	action = handlepredatflt(param);
+	if(action == HANDLED){
+		RETURN(0);
+	}
+	if(action != PASS) RETURN(19);
 	action = handledatfltsrv(param,  &param->srvbuf, (int *)&param->srvbufsize, 0, (int *)&param->srvinbuf);
 	param->nolongdatfilter = 1;
 	if(action == HANDLED){
@@ -990,11 +1037,9 @@ for(;;){
 		"Proxy-support: Session-Based-Authentication\r\n"
 		"Connection: Proxy-support\r\n"
 	 );
-	 if(!param->srv->transparent){
+	 if(!param->srv->transparent && res>=200){
 		if(ckeepalive <= 1) sprintf((char*)buf+strlen((char *)buf), "Connection: %s\r\n", 
-		(hascontent && ckeepalive)?"Keep-Alive":"Close");
-		if(!param->transparent)sprintf((char*)buf+strlen((char *)buf), "Proxy-Connection: %s\r\n", 
-		(hascontent && ckeepalive)?"Keep-Alive":"Close");
+		(hascontent && ckeepalive)?"keep-alive":"close");
 	 }
 	 sprintf((char*)buf + strlen((char *)buf), "\r\n");
 	 if((socksend(param->clisock, buf, (int)strlen((char *)buf), conf.timeouts[STRING_S])) != (int)strlen((char *)buf)) {
@@ -1034,7 +1079,7 @@ for(;;){
 		}
 		if(param->chunked != 2){
 			param->waitserver64 = contentlength64;
-		 	if((res = sockmap(param, conf.timeouts[CONNECTION_S])) != 98){
+		 	if((res = mapsocket(param, conf.timeouts[CONNECTION_S])) != 98){
 				RETURN(res);
 			}
 	 		param->waitserver64 = 0;
@@ -1042,13 +1087,13 @@ for(;;){
         } while(param->chunked);
  }
  if(isconnect && res == 200 && param->operation){
-	RETURN (sockmap(param, conf.timeouts[CONNECTION_S]));
+	RETURN (mapsocket(param, conf.timeouts[CONNECTION_S]));
  }
  else if(isconnect){
 	ckeepalive = keepalive = 1;
  }
  else if(!hascontent && !param->chunked) {
-	RETURN(sockmap(param, conf.timeouts[CONNECTION_S]));
+	RETURN(mapsocket(param, conf.timeouts[CONNECTION_S]));
  }
  contentlength64 = 0;
 REQUESTEND:
